
// local headers
#import "Shoot.h"
#import "Explosion.h"
#import "PlayerGun.h"
#import "PlayerBullet.h"
#import "SBrick.h"
#import "SCloud.h"
#import "GOLetter.h"
#import "SoundMgr.h"

// XoX headers
#import "BackView.h"
#import "ActorMgr.h"
#import "CacheManager.h"
#import "xoxDefs.h"
#import "Thinker.h"

int SHOOTSND;

@implementation Shoot

// invoked only by the actor manager
- _createLevel:(int)lev
{
    int i;
    
    player1 = 
	    [actorMgr newActor:(int)[PlayerGun class] for:self tag:EVIL];
    player2 = 
	    [actorMgr newActor:(int)[PlayerGun class] for:self tag:GOOD];

    currentPlayer = player1;

    if ([actorMgr gameStatus] == GAME_DYING)
        [[mainView window] setTitle:"Shoot! - Game Over"];
    else
        [[mainView window] setTitle:"Shoot!"];
    
    shootOff = YES;
    //shootOff = NO;
    windSpeed = randBetween(-10,10);
    startPow1 = startPow2 = -1;
     
    for (i=3; i>=0; i--)
    	[actorMgr newActor:(int)[SCloud class] for:self tag:i];

    [actorMgr newActor:(int)[SBrick class] for:self tag:0];

    [self perform:@selector(updatePower) with:nil
    	afterDelay:0.3 cancelPrevious:YES];
	
    [pow1 setFloatValue:0.0];
    [pow2 setFloatValue:0.0];
    if (lev == 1) {
	[score1 setIntValue:0];
	[score2 setIntValue:0];
    }

    // this is kind of a hack.  if the game is over and the window is resized,
    // we want _createLevel to be called so that it will create a new level
    // that makes sense for the size of the screen.  However, there's no way
    // to do that and to keep the game over text on the screen.  In addition,
    // there's no way to grab the game over letters and just drop them back on
    // the screen after the actor manager has removed them.  Oh, well.
    // So the solution is to restart the game over animation on resize.
    if ([actorMgr gameStatus] == GAME_DYING)
        [GOLetter gameOver:self];

    return self;
}

- updatePower
{
    int val;
    
    if (startPow1 != -1) {
	val = (timeInMS - startPow1);
	[pow1 setIntValue:val];
    }
    if (startPow2 != -1) {
	val = (timeInMS - startPow2);
	[pow2 setIntValue:val];
    }
    [self perform:@selector(updatePower) with:nil
    	afterDelay:0.3 cancelPrevious:YES];
    return self;
}

- (float)windSpeed
{
    return windSpeed;
}

- currentPlayer
{
    return currentPlayer;
}

- infoView
{
    return infoView;
}

- didActivate:(Actor *)theActor
{
    if ([theActor isKindOf:[Explosion class]]) {
	[soundMgr playSound:EXP3SND at:0.5];
    }
    return self;
}

- didRetire:(Actor *)theActor
{
    if ([theActor isKindOf:[PlayerBullet class]]) {
	currentPlayer = ((lastPlayer == player1) ? player2 : player1);
    } else if ([theActor isKindOf:[Explosion class]]) {
	if ([score1 intValue] >= 10 || [score2 intValue] >= 10) {
            if ([actorMgr gameStatus] != GAME_DYING) {
		[GOLetter gameOver:self];
                [[mainView window] setTitle:"Shoot! - Game Over"];
            }
	    [actorMgr setGameStatus: GAME_DYING];
	}
	if ([actorMgr gameStatus] != GAME_DYING)
	    [actorMgr requestLevel:2];
    } else if ([theActor isKindOf:[PlayerGun class]]) {
	if (theActor->alliance == GOOD) {
	    [score1 setIntValue:[score1 intValue]+1];
	} else {
	    [score2 setIntValue:[score2 intValue]+1];
	}
    }
    return self;
}

- keyDown:(NXEvent *)theEvent
{
    if (theEvent->data.key.repeat > 0) 
	return self;

    switch (theEvent->data.key.charCode)
    {
	case ',':		// move left
	    if (!shootOff && (currentPlayer != player1))
		break;
	    [leftVal1 keyDownAt:theEvent->time];
	    break;
	case '2':		// move left
	    if (!shootOff && (currentPlayer != player2))
		break;
	    [leftVal2 keyDownAt:theEvent->time];
	    break;
	case '.':		// move right
	    if (!shootOff && (currentPlayer != player1))
		break;
	    [rightVal1 keyDownAt:theEvent->time];
	    break;
	case '3':		// move right
	    if (!shootOff && (currentPlayer != player2))
		break;
	    [rightVal2 keyDownAt:theEvent->time];
	    break;
	case ' ':		// fire
	    if (!shootOff && (currentPlayer != player1))
		break;
	    [shootVal1 keyDownAt:theEvent->time];
	    startPow1 = timeInMS;
	    break;
	case '0':		// fire
	    if (!shootOff && (currentPlayer != player2))
		break;
	    [shootVal2 keyDownAt:theEvent->time];
	    startPow2 = timeInMS;
	    break;
    }
    return self;
}

- keyUp:(NXEvent *)theEvent
{
    switch (theEvent->data.key.charCode)
    {
	case ',':		// move left
	    if (!shootOff && (currentPlayer != player1))
		break;
	    [leftVal1 keyUpAt:theEvent->time];
	    break;
	case '2':		// move left
	    if (!shootOff && (currentPlayer != player2))
		break;
	    [leftVal2 keyUpAt:theEvent->time];
	    break;
	case '.':		// move right
	    if (!shootOff && (currentPlayer != player1))
		break;
	    [rightVal1 keyUpAt:theEvent->time];
	    break;
	case '3':		// move right
	    if (!shootOff && (currentPlayer != player2))
		break;
	    [rightVal2 keyUpAt:theEvent->time];
	    break;
	case ' ':		// fire
	    if (!shootOff && (currentPlayer != player1))
		break;
	    [shootVal1 keyUpAt:theEvent->time];
	    [player1 fire:[shootVal1 val]*4];
	    lastPlayer = currentPlayer;
	    currentPlayer = nil;
	    [shootVal1 postOneStep];
	    startPow1 = -1;
	    break;
	case '0':		// fire
	    if (!shootOff && (currentPlayer != player2))
		break;
	    [shootVal2 keyUpAt:theEvent->time];
	    [player2 fire:[shootVal2 val]*4];
	    lastPlayer = currentPlayer;
	    currentPlayer = nil;
	    [shootVal2 postOneStep];
	    startPow2 = -1;
	    break;
    }
    return self;
}

- scenarioSelected
{
    [keyTimerList addObject:leftVal1];
    [keyTimerList addObject:rightVal1];
    [keyTimerList addObject:leftVal2];
    [keyTimerList addObject:rightVal2];
    return self; 
}

- scenarioDeselected
{	
    [[gcontentView window] setBackgroundGray:NX_BLACK];
    return self; 
}

- init
{
    char path[256];

    [super init];

    if ([[NXBundle bundleForClass:[self class]]
	    getPath:path
	    forResource:"shoot"
	    ofType:"nib"])
    {
	[NXApp loadNibFile:path
		owner:self
		withNames:NO
		fromZone:[self zone]];
    }

    leftVal1 = [[KeyTimer allocFromZone:[self zone]] init];
    rightVal1 = [[KeyTimer allocFromZone:[self zone]] init];
    leftVal2 = [[KeyTimer allocFromZone:[self zone]] init];
    rightVal2 = [[KeyTimer allocFromZone:[self zone]] init];
    shootVal1 = [[KeyTimer allocFromZone:[self zone]] init];
    shootVal2 = [[KeyTimer allocFromZone:[self zone]] init];
    [leftVal1 setDelegate:rightVal1];
    [rightVal1 setDelegate:leftVal1];
    [leftVal2 setDelegate:rightVal2];
    [rightVal2 setDelegate:leftVal2];

    SHOOTSND = [soundMgr addSound:"35" sender:self];
    [[NXApp delegate] addImageResource:"explosionM" for:[Explosion class]];
    [[NXApp delegate] addSoundResource:EXP3SND];
    [[NXApp delegate] addSoundResource:SHOOTSND];
    [[NXApp delegate] addSoundResource:FUTILITYSND];

    return self;
}

- (COLLISION_PARADIGM)collisionParadigm
{
    return GOOD_V_EVIL;
}

- newSize:(NXSize *)s
{
    NXRect r, r2;
    [pow2 getFrame:&r];
    [pow2 moveTo:(s->width - NX_WIDTH(&r) - 10) :10];

    [pow1 getFrame:&r2];
    [score1 moveTo:NX_MAXX(&r2) + 10 :10];
    [pow2 getFrame:&r2];
    [score2 getFrame:&r];
    [score2 moveTo:(NX_X(&r2) - NX_WIDTH(&r) - 10) :10];

    [actorMgr requestLevel:2];

    return self;
}

- tile
{
    NXRect r, r2;

    [[gcontentView window] setBackgroundGray:NX_LTGRAY];
    [gcontentView getBounds:&r];
    r.size.height -= 40;
    r.origin.y += 40;
    [abackView setFrame:&r];
    [gcontentView getBounds:&r];
    r.size.height = 40;
    [scoreView setFrame:&r];
    [gcontentView addSubview:abackView];
    [gcontentView addSubview:scoreView];
    [gcontentView addSubview:pow1];
    [gcontentView addSubview:pow2];
    [gcontentView addSubview:score1];
    [gcontentView addSubview:score2];
    [pow1 moveTo:10 :10];
    [pow2 getFrame:&r2];
    [pow2 moveTo:(NX_WIDTH(&r) - NX_WIDTH(&r2) - 10) :10];

    [pow1 getFrame:&r2];
    [score1 moveTo:NX_MAXX(&r2) + 10 :10];
    [pow2 getFrame:&r2];
    [score2 getFrame:&r];
    [score2 moveTo:(NX_X(&r2) - NX_WIDTH(&r) - 10) :10];
    
    return abackView;
}

- (float)rotValFor:(int)who
{
    float rot = 0;
    if (who) {
	if (shootOff || (currentPlayer == player1))
	    rot = [leftVal1 val] - [rightVal1 val];
    } else {
	if (shootOff || (currentPlayer == player2))
	    rot = [leftVal2 val] - [rightVal2 val];
    }
    return rot;
}

@end
